#if !defined  HAVE_BINARY_DEBRUIJN_H__
#define       HAVE_BINARY_DEBRUIJN_H__


#include "fxttypes.h"
#include "comb/binary-necklace.h"
#include "bits/tinyfactors.h"


class binary_debruijn : public binary_necklace
// Binary De Bruijn sequence
{
public:
    ulong nt_, k_;
    ulong dq_;
    ulong tfb_; // tiny factor bits

public:
    binary_debruijn(ulong n)
        : binary_necklace(n)
    {
        first();
    }

    ~binary_debruijn()  { ; }

    void first(ulong n=0)
    {
        binary_necklace::first();
        nt_ = 1;
        k_ = 0;
        dq_ = 0;
        tfb_ = tiny_factors_tab[binary_necklace::n_];
    }

    int done()  const  { return  dq_; }

    ulong next_bit()  // return new element (0 or 1)
    {
        if ( 0==nt_ )  next_string();
        --nt_;
        ++k_;
        return  a_[ k_ ];
    }

    void next_string()  // make new string
    {
        do
        {
            binary_necklace::next();
            if ( 0==j_ )  { dq_ = 1; return; }
        }
        while ( 0 == ( (tfb_>>j_) & 1 ) );

        nt_ = j_;
        k_ = 0;
    }

    ulong word()  const
    {
        ulong t = 0;
        ulong ib = 1UL << (n_ - 1);
        for (ulong i=1;  i<=n_;  ++i, ib>>=1)  if ( a_[i] )  t |= ib;
        return t;
    }

    void make_seq(ulong *f, ulong n)
    {
        first();
        for (ulong k=0; k<n; ++k)  f[k] = next_bit();
    }

    ulong next_bit_into_word(ulong x)
    {
        x = ( (x<<1) & (n_-1) );
        x |= next_bit();
        return  x;
    }
};
// -------------------------


#endif  // !defined HAVE_BINARY_DEBRUIJN_H__
