#if !defined  HAVE_PERM_HEAP2_SWAPS_H__
#define       HAVE_PERM_HEAP2_SWAPS_H__


#include "aux0/swap.h"
#include "comb/comb-print.h"
#include "fxttypes.h"


#define PERMHEAP2_SWAPS_FIXARRAYS // use arrays instead of pointers (speedup)


class perm_heap2_swaps
// Swaps for Gray code for permutations (of at least three objects).
// Algorithm following B.R.Heap "Permutations by interchanges" (1963)
// Optimized implementation, very fast.
{
public:
#ifndef PERMHEAP2_SWAPS_FIXARRAYS
    ulong *d_;  // mixed radix digits with radix = [2, 3, 4, 5, ..., n-1, (sentinel=-1)]
#else
    ulong d_[32];
#endif
    ulong n_;   // permutations of n elements
    ulong sw1_, sw2_;   // indices of swapped elements
    ulong ct_;  // count 5,4,3,2,1,(0); nonzero ==> easy cases

public:
    perm_heap2_swaps(ulong n)  // must have n>=3
    {
        n_ = n;
#ifndef PERMHEAP2_SWAPS_FIXARRAYS
        d_ = new ulong[n_];
#endif
        // d[0] and d[1] are unused
        d_[n-1] = -1UL;  // sentinel
        first();
    }

    ~perm_heap2_swaps()
    {
#ifndef PERMHEAP2_SWAPS_FIXARRAYS
        delete [] d_;
#endif
    }

    void first()
    {
        for (ulong k=0; k<n_-1; ++k)  d_[k] = 0;
        sw1_=0;  sw2_=0;
        ct_ = 5;
    }

    bool next()
    {
        if ( ct_!=0 )  // easy case(s)
        {
            --ct_;
            sw1_ = 1 + (ct_ & 1);  // == 1,2,1,2,1
            sw2_ = 0;
            return true;
        }
        else
        {
            ct_ = 5;  // reset counter

            // increment mixed radix number:
            ulong j = 2;
            while ( d_[j]==j+1 )  { d_[j]=0; ++j; }

            // j==n-1 for last permutation
            if ( j!=n_-1 ) // only if no overflow
            {
                ulong k = j+1;
                ulong x = ( (k&1)  ? d_[j] : 0);
                sw1_ = k;  sw2_ = x;

                ++d_[j];
                return true;
            }
            else  return false;
        }
    }

    void get_swap(ulong &s1, ulong &s2)  const  { s1=sw1_; s2=sw2_; }
};
// -------------------------

#undef PERMHEAP2_SWAPS_FIXARRAYS


#endif  // !defined HAVE_PERM_HEAP2_SWAPS_H__
