#if !defined  HAVE_PERM_HEAP2_H__
#define       HAVE_PERM_HEAP2_H__


#include "aux0/swap.h"
#include "comb/comb-print.h"
#include "fxttypes.h"


//#define PERMHEAP2_FIXARRAYS // use arrays instead of pointers (not a win)


class perm_heap2
// Gray code for permutations (of at least three objects).
// Algorithm following B.R.Heap "Permutations by interchanges" (1963)
// Optimized implementation, very fast.
{
public:
#ifndef PERMHEAP2_FIXARRAYS
    ulong *d_;  // mixed radix digits with radix = [2, 3, 4, 5, ..., n-1, (sentinel=-1)]
    ulong *p_;  // permutation
#else
    ulong d_[32];
    ulong p_[32];  // permutation
#endif // PERMHEAP2_FIXARRAYS

    ulong n_;   // permutations of n elements
    ulong sw1_, sw2_;   // indices of swapped elements
    ulong ct_;  // count 5,4,3,2,1,(0); nonzero ==> easy cases

public:
    perm_heap2(ulong n)  // must have n>=3
    {
        n_ = n;
#ifndef PERMHEAP2_FIXARRAYS
        d_ = new ulong[n_];
        p_ = new ulong[n_];
#endif
        // d[0] and d[1] are unused
        d_[n-1] = -1UL;  // sentinel
        first();
    }

    ~perm_heap2()
    {
#ifndef PERMHEAP2_FIXARRAYS
        delete [] p_;
        delete [] d_;
#endif
    }

    const ulong * data()  const  { return p_; }

    void first()
    {
        for (ulong k=0; k<n_-1; ++k)  d_[k] = 0;
        for (ulong k=0; k<n_; ++k)  p_[k] = k;
        sw1_=0;  sw2_=0;
        ct_ = 5;
    }

    bool next()
    {
        if ( ct_!=0 )  // easy case(s)
        {
            --ct_;
            sw1_ = 1 + (ct_ & 1);  // == 1,2,1,2,1
            sw2_ = 0;
            swap2(p_[sw1_], p_[sw2_]);
            return true;
        }
        else
        {
            ct_ = 5;  // reset counter

            // increment mixed radix number:
            ulong j = 2;
            while ( d_[j]==j+1 )  { d_[j]=0; ++j; }

            // j==n-1 for last permutation
            if ( j!=n_-1 ) // only if no overflow
            {
                ulong k = j+1;
                ulong x = ( (k&1)  ? d_[j] : 0);
                swap2(p_[k], p_[x]);
                sw1_ = k;  sw2_ = x;

                ++d_[j];
                return true;
            }
            else  return false;
        }
    }

    void get_swap(ulong &s1, ulong &s2)  const  { s1=sw1_; s2=sw2_; }

    void print(const char *bla, bool dfz=false)  const
    // If dfz is true then Dots are printed For Zeros.
    {
        print_perm(bla, data(), n_, dfz);
    }
};
// -------------------------

#undef PERMHEAP2_FIXARRAYS

#endif  // !defined HAVE_PERM_HEAP2_H__
