#if !defined  HAVE_ARITHTRANSFORM_H__
#define       HAVE_ARITHTRANSFORM_H__


#include "fxttypes.h"
#include "restrict.h"


template <typename Type>
void arith_transform_plus(Type *f, ulong ldn)
// Arithmetic Transform (positive sign).
// Radix-2 decimation In Frequency (DIF) algorithm.
// Inverse of arith_transform_minus().
{
    const ulong n = (1UL<<ldn);
    for (ulong ldm=ldn; ldm>=1; --ldm)
    {
        const ulong m = (1UL<<ldm);
        const ulong mh = (m>>1);
        for (ulong r=0; r<n; r+=m)
        {
            ulong t1 = r;
            ulong t2 = r+mh;
            for (ulong j=0;  j<mh;  ++j, ++t1, ++t2)
            {
                Type u = f[t1];
                Type v = f[t2];
                f[t1] = u;
                f[t2] = u + v;
            }
        }
    }
}
// -------------------------

template <typename Type>
void arith_transform_minus(Type *f, ulong ldn)
// Arithmetic Transform (negative sign).
// Radix-2 decimation In Frequency (DIF) algorithm.
// Inverse of arith_transform_plus().
{
    const ulong n = (1UL<<ldn);
    for (ulong ldm=ldn; ldm>=1; --ldm)
    {
        const ulong m = (1UL<<ldm);
        const ulong mh = (m>>1);
        for (ulong r=0; r<n; r+=m)
        {
            ulong t1 = r;
            ulong t2 = r+mh;
            for (ulong j=0;  j<mh;  ++j, ++t1, ++t2)
            {
                Type u = f[t1];
                Type v = f[t2];
                f[t1] = u;
                f[t2] = v - u;
            }
        }
    }
}
// -------------------------

template <typename Type>
inline void arith_transform(Type *f, ulong ldn, int is)
{
    if ( is>0 )  arith_transform_plus(f, ldn);
    else         arith_transform_minus(f, ldn);
}
// -------------------------


template <typename Type>
void transposed_arith_transform_plus(Type *f, ulong ldn)
// Transposed arithmetic Transform (positive sign).
// Radix-2 decimation In Frequency (DIF) algorithm.
// Inverse of transposed_arith_transform_minus().
{
    const ulong n = (1UL<<ldn);
    for (ulong ldm=ldn; ldm>=1; --ldm)
    {
        const ulong m = (1UL<<ldm);
        const ulong mh = (m>>1);
        for (ulong r=0; r<n; r+=m)
        {
            ulong t1 = r;
            ulong t2 = r+mh;
            for (ulong j=0;  j<mh;  ++j, ++t1, ++t2)
            {
                Type u = f[t1];
                Type v = f[t2];
                f[t1] = u + v;
                f[t2] = v;
            }
        }
    }
}
// -------------------------

template <typename Type>
void transposed_arith_transform_minus(Type *f, ulong ldn)
// Transposed arithmetic Transform (negative sign).
// Radix-2 decimation In Frequency (DIF) algorithm.
// Inverse of transposed_arith_transform_plus().
{
    const ulong n = (1UL<<ldn);
    for (ulong ldm=ldn; ldm>=1; --ldm)
    {
        const ulong m = (1UL<<ldm);
        const ulong mh = (m>>1);
        for (ulong r=0; r<n; r+=m)
        {
            ulong t1 = r;
            ulong t2 = r+mh;
            for (ulong j=0;  j<mh;  ++j, ++t1, ++t2)
            {
                Type u = f[t1];
                Type v = f[t2];
                f[t1] = u - v;
                f[t2] = v;
            }
        }
    }
}
// -------------------------

template <typename Type>
inline void transposed_arith_transform(Type *f, ulong ldn, int is)
{
    if ( is>0 )  transposed_arith_transform_plus(f, ldn);
    else         transposed_arith_transform_minus(f, ldn);
}
// -------------------------


#endif  // !defined HAVE_ARITHTRANSFORM_H__
