#if !defined  HAVE_OR_CONVOLUTION_H__
#define       HAVE_OR_CONVOLUTION_H__


#include "walsh/arithtransform.h"

#include "fxttypes.h"
#include "restrict.h"



template <typename Type>
inline void or_convolution(Type * restrict f, Type * restrict g, ulong ldn)
// Compute the OR-convolution h[] of f[] and g[]:
//  h[k] = sum(i | j == k,  f[i]*g[j])
// f[] and g[] must not overlap.
// Result written to g[].
{
    arith_transform_plus(f, ldn);
    arith_transform_plus(g, ldn);
    const ulong n = (1UL<<ldn);
    for (ulong k=0; k<n; ++k)  g[k] *= f[k];
    arith_transform_minus(g, ldn);
}
// -------------------------

template <typename Type>
inline void or_auto_convolution(Type *f, ulong ldn)
// Compute the self- OR-convolution h[] of f[]:
//  h[k] = sum(i | j == k,  f[i]*f[j])
// Result written to f[].
{
    arith_transform_plus(f, ldn);
    const ulong n = (1UL<<ldn);
    for (ulong k=0; k<n; ++k)  f[k] *= f[k];
    arith_transform_minus(f, ldn);
}
// -------------------------


#endif  // !defined HAVE_OR_CONVOLUTION_H__
